#!/usr/bin/env python

import ctypes
import logging
import os
import struct
import sys

__copyright__ = "Copyright 2017, Aleksei Ilin"
__author__ = "Aleksei Ilin"
# __email__ = ""


DEBUG = False


DLL_PREFIX = os.path.dirname(os.path.abspath(__file__))


def str2bool(s):
    try:
        return int(s) == 1
    except:
        return str(s).lower().startswith('t')


def iteritems(dct):
    """ Iterate over dict items """

    if sys.version_info < (3, 0, 0):
        return dct.iteritems()
    else:
        return iter(dct.items())


def is_windows_64bit():
    if 'PROCESSOR_ARCHITEW6432' in os.environ:
        return True
    return os.environ['PROCESSOR_ARCHITECTURE'].endswith('64')


def is_python_64bit():
    return struct.calcsize("P") == 8


class DllObject(object):
    """ Loads / unloads specified DLL """

    def __init__(self, ctypes_conv, dll_name):
        self._ctypes_conv = ctypes_conv
        self._dll_name = dll_name
        self._dll_ctypes = self._ctypes_conv.LoadLibrary(self.dll_name)

    @property
    def ctypes_conv(self):
        return self._ctypes_conv

    @property
    def dll_name(self):
        return self._dll_name

    @property
    def dll_ctypes(self):
        return self._dll_ctypes


class DllFunction(object):
    """ Function object of an ordinary DLL """

    def __init__(self, dll_object, name):
        self._dll_object = dll_object
        self._name = name

        self._func = getattr(self._dll_object.dll_ctypes, name)

    def call(self, *args):
        return self._func(*args)

    @property
    def name(self):
        return self._name


def dump_struct(s):
    return '\n'.join(s.to_list_pair())


class Struct(ctypes.Structure):
    def __str__(self):
        return self.dump()

    def dump(self):
        return dump_struct(self)

    def to_list_pair(self):
        lst = list()
        for field in self._fields_:
            name = field[0]
            if DEBUG or not name.startswith('_reserved'):
                lst.append((name, getattr(self, name)))
        return lst


class FirmwareInfo(Struct):
    """ Firmware info C structure representation """

    _pack_ = 1
    _fields_ = [("len", ctypes.c_uint16),
                ("a", ctypes.c_uint16),
                ("b", ctypes.c_uint16),
                ("c", ctypes.c_uint16),
                ("d", ctypes.c_uint16)]

    def dump(self):
        def to_hex(i):
            return str(hex(i)[2:]).upper().rjust(4, '0')

        return '-'.join([to_hex(self.a), to_hex(self.b), to_hex(self.c), to_hex(self.d)])


class ChargeMode(object):
    """ Enumeration object for Charge Mode """

    MODE_Auto = 0
    MODE_NonAuto = 1

    def __init__(self, value):
        self.value = value

    def __str__(self):
        return self.dump()

    def __int__(self):
        return self.value

    def dump(self):
        if self.MODE_Auto == self.value:
            return 'Auto'
        if self.MODE_NonAuto == self.value:
            return 'Non auto'
        return 'Unknown: %d' % self.value


class SmartBatteryStatus(Struct):
    """ Smart battery status C structure representation """

    # chemistry types
    CHEMISTRY_UNKNOWN = -1
    CHEMISTRY_VALUE_0 = 0x00001
    CHEMISTRY_NiMH = 0x00010
    CHEMISTRY_LiIon = 0x00100
    CHEMISTRY_LiPol = 0x01000
    CHEMISTRY_AgZn = 0x10000

    # battery health
    HEALTH_Good = 0
    HEALTH_Fair = 10
    HEALTH_Poor = 20

    # battery health boundary
    HEALTHBOUNDARY_Good = 0
    HEALTHBOUNDARY_Poor = 1

    # charge status
    CHARGE_NoCharging = 0
    CHARGE_Charging = 1
    CHARGE_Discharging = 2

    _pack_ = 1
    _fields_ = [("struct_sz", ctypes.c_int),

                ("set_charge_status", ctypes.c_int),
                ("charge_status", ctypes.c_int),

                ("set_ac_status", ctypes.c_int),
                ("ac_status", ctypes.c_int),

                ("condition", ctypes.c_int),

                ("set_dword18", ctypes.c_int),
                ("dword1c", ctypes.c_int),

                ("set_remaining_percentage", ctypes.c_int),
                ("remaining_percentage", ctypes.c_int),

                ("set_remaining_capacity", ctypes.c_int),
                ("remaining_capacity", ctypes.c_int),

                ("set_remaining_time", ctypes.c_int),
                ("remaining_time", ctypes.c_int),

                ("set_trickle_charge", ctypes.c_int),
                ("trickle_charge", ctypes.c_int),

                ("set_voltage", ctypes.c_int),
                ("voltage", ctypes.c_int),

                ("set_wattage", ctypes.c_int),
                ("wattage", ctypes.c_int),

                ("set_temperature", ctypes.c_int),
                ("temperature", ctypes.c_int),

                ("set_cycle_count", ctypes.c_int),
                ("cycle_count", ctypes.c_int),

                ("set_fcc_ratio", ctypes.c_int),
                ("fcc_ratio", ctypes.c_int),

                ("set_design_capacity", ctypes.c_int),
                ("design_capacity", ctypes.c_int),

                ("set_design_voltage", ctypes.c_int),
                ("design_voltage", ctypes.c_int),

                ("set_chemistry_type", ctypes.c_int),
                ("chemistry_type", ctypes.c_int),

                ("set_serial_number", ctypes.c_int),
                ("serial_number", ctypes.c_int),

                ("set_manufacturing_date", ctypes.c_int),
                ("manufacturing_date", ctypes.c_char * 12),

                ("set_dword98", ctypes.c_int),
                ("word9c", ctypes.c_int16),
                ("word9e", ctypes.c_int16),

                ("set_battery_manufacturer", ctypes.c_int),
                ("battery_manufacturer", ctypes.c_char * 15),

                ("ireserved1", ctypes.c_int),
                ("creserved1", ctypes.c_char * 8),

                ("set_battery_model", ctypes.c_int),
                ("battery_model", ctypes.c_char * 15),

                ("ireserved2", ctypes.c_int),
                ("creserved2", ctypes.c_char * 8),

                ("set_barcode_number", ctypes.c_int),
                ("barcode_number", ctypes.c_char * 15),

                ("ireserved3", ctypes.c_int),
                ("creserved3", ctypes.c_char * 8),

                ("set_first_used_date", ctypes.c_int),
                ("first_used_date", ctypes.c_char * 12),

                ("set_condition_date", ctypes.c_int),
                ("condition_date", ctypes.c_char * 12),

                ("set_health_status", ctypes.c_int),
                ("health_status", ctypes.c_int),
                ("health_status_boundary", ctypes.c_int),

                ("set_ac_wattage", ctypes.c_int),
                ("ac_wattage", ctypes.c_int),

                ("dword131", ctypes.c_int)]

    def __init__(self):
        Struct.__init__(self)
        self.struct_sz = ctypes.sizeof(self)

    def descr_chemistry_type(self):
        if 1 != self.set_chemistry_type:
            return None
        if self.CHEMISTRY_LiIon == self.chemistry_type:
            return 'Li-Ion'
        if self.CHEMISTRY_LiPol == self.chemistry_type:
            return 'Li-Pol'
        if self.CHEMISTRY_NiMH == self.chemistry_type:
            return 'NiMh'
        if self.CHEMISTRY_AgZn == self.chemistry_type:
            return 'AgZn'
        return 'Unknown: %d' % self.chemistry_type

    def descr_battery_health(self):
        if 1 != self.set_health_status:
            return None
        if self.HEALTH_Good == self.health_status:
            return 'Good'
        if self.HEALTH_Fair == self.health_status:
            return 'Fair'
        if self.HEALTH_Poor == self.health_status:
            return 'Poor'
        return 'Unknown: %d' % self.health_status

    def descr_battery_health_boundary(self):
        if 1 != self.set_health_status:
            return None
        if self.HEALTHBOUNDARY_Good == self.health_status_boundary:
            return 'Good'
        if self.HEALTHBOUNDARY_Poor == self.health_status_boundary:
            return 'Poor'
        return 'Unknown: %d' % self.health_status_boundary

    def descr_charge_status(self):
        if 1 != self.set_charge_status:
            return None
        if self.CHARGE_NoCharging == self.charge_status:
            return 'No activity'
        if self.CHARGE_Charging == self.charge_status:
            return 'Charging'
        if self.CHARGE_Discharging == self.charge_status:
            return 'Discharging'
        return 'Unknown: %d' % self.charge_status

    def to_list_pair(self):
        lst = list()

        def add_to_dict(name, value, value_check):
            if 1 == value_check:
                lst.append((name, str(value)))

        add_to_dict('Manufacturer', self.battery_manufacturer, self.set_battery_manufacturer)
        add_to_dict('Part number (FRU)', self.battery_model, self.set_battery_model)
        add_to_dict('Barcode number', self.barcode_number, self.set_barcode_number)
        add_to_dict('Serial number', self.serial_number, self.set_serial_number)

        add_to_dict('Chemistry type', self.descr_chemistry_type(), self.set_chemistry_type)

        add_to_dict('Design voltage, V', float(self.design_voltage) / 1000, self.set_design_voltage)

        add_to_dict('Manufacturing date', self.manufacturing_date, self.set_manufacturing_date)
        add_to_dict('First used date', self.first_used_date, self.set_first_used_date)
        add_to_dict('Conditioning date', self.condition_date, self.set_condition_date)

        add_to_dict('AC status', self.ac_status, self.set_ac_status)
        add_to_dict('AC wattage, W', self.ac_wattage, self.set_ac_wattage)

        add_to_dict('Battery condition', self.condition, 1)
        add_to_dict('Battery health', self.descr_battery_health(), self.set_health_status)
        add_to_dict('Health boundary', self.descr_battery_health_boundary(), self.set_health_status)
        add_to_dict('Cycle count', self.cycle_count, self.set_cycle_count)
        add_to_dict('Temperature, C', self.temperature, self.set_temperature)
        add_to_dict('Voltage, V', float(self.voltage) / 1000, self.set_voltage)

        add_to_dict('Charge status', self.descr_charge_status(), self.set_charge_status)
        add_to_dict('Trickle charge', self.trickle_charge, self.set_trickle_charge)

        add_to_dict('Design capacity, W', float(self.design_capacity) / 1000, self.set_design_capacity)
        add_to_dict('Full charge capacity, W', float(self.fcc_ratio) / 1000, self.set_fcc_ratio)
        add_to_dict('Remaining capacity, W', float(self.remaining_capacity) / 1000, self.set_remaining_capacity)
        add_to_dict('Remaining capacity, %', self.remaining_percentage, self.set_remaining_percentage)

        add_to_dict('Remaining time', '%d h %d min' % divmod(self.remaining_time, 60), self.set_remaining_time)
        add_to_dict('Wattage, W', float(abs(self.wattage)) / 100, self.set_wattage)

        add_to_dict('Current, A', (float(abs(self.wattage)) / 100) / (float(self.voltage) / 1000),
                    1 if (1 == self.set_voltage) and (1 == self.set_wattage) else 0)

        if DEBUG:
            add_to_dict('UNKNWN. DWORD 1C', self.dword1c, self.set_dword18)
            add_to_dict('UNKNWN. WORD 9C', self.word9c, self.set_dword98)
            add_to_dict('UNKNWN. WORD 9E', self.word9e, self.set_dword98)
            add_to_dict('UNKNWN. DWORD 131', self.dword131, 1)

        return lst


class ChargeCapacityThreshold(Struct):
    """ Charge capacity threshold C structure representation """

    _pack_ = 1
    _fields_ = [("set_battery_main", ctypes.c_int),
                ("set_battery_second", ctypes.c_int),
                ("percentage", ctypes.c_int),
                ("_reserved", ctypes.c_int * 5)]


class DischargeStatus(Struct):
    """ Discharge status C structure representation """

    _pack_ = 1
    _fields_ = [("support_discharge", ctypes.c_int),
                ("support_separate_batteries", ctypes.c_int),
                ("support_break_on_ac_detach", ctypes.c_int),
                ("enable_discharge", ctypes.c_int),
                ("enable_break_on_ac_detach", ctypes.c_int),
                ("_reserved", ctypes.c_int * 3)]


class InhibitChargeStatus(Struct):
    """ Inhibit charge status C structure representation """

    _pack_ = 1
    _fields_ = [("main_battery", ctypes.c_int),
                ("second_battery", ctypes.c_int),
                ("dw8", ctypes.c_int),
                ("enable_inhibition", ctypes.c_int),
                ("inhibition_period", ctypes.c_int),
                ("_reserved", ctypes.c_int * 3)]


class BatteryMgr(object):
    @staticmethod
    def _GetChargeCapacityThreshold(battery_id, cct):
        if battery_id == 1:
            if cct.set_battery_main:
                return cct.percentage
        elif battery_id == 2:
            if cct.set_battery_second:
                return cct.percentage

    @staticmethod
    def _delay_execution():
        from time import sleep
        sleep(0.5)  # Time in seconds.

    @staticmethod
    def _log_result(func, result):
        logging.debug('Result of %s: %d' % (func.name, result))

    def __init__(self):
        dll_name = 'PWMIF64V.DLL' if is_python_64bit() else 'PWMIF32V.DLL'
        dll_path = DLL_PREFIX + '\\' + ('x64' if is_python_64bit() else 'x86') + '\\' + dll_name

        paths = str(os.environ['PATH']).split(os.pathsep)
        if not os.path.dirname(dll_path) in paths:
            os.environ['PATH'] = os.path.dirname(dll_path) + os.pathsep + os.environ['PATH']

        self._dll_obj = DllObject(ctypes.windll, dll_path)

    def SM_GetIntelCpuArchitecture(self):
        # typedef int (*PSM_GetIntelCpuArchitecture)(int* pOutArch);
        func = DllFunction(self._dll_obj, 'SM_GetIntelCpuArchitecture')
        arch = ctypes.c_int(0)
        result = func.call(ctypes.byref(arch))
        BatteryMgr._log_result(func, result)
        return arch.value if 0 == int(result) else 0

    def SM_IsUEFI(self):
        # typedef int (*PSM_IsUEFI)();
        func = DllFunction(self._dll_obj, 'SM_IsUEFI')
        result = func.call()
        BatteryMgr._log_result(func, result)
        return result

    def SM_GetFirmwareInformation(self, battery_id):
        # typedef int (*PSM_GetFirmwareInformation)(int batteryid, FirmwareInfo* pOutInfo);
        fi = FirmwareInfo()
        func = DllFunction(self._dll_obj, 'SM_GetFirmwareInformation')
        result = func.call(battery_id, ctypes.byref(fi))
        BatteryMgr._log_result(func, result)
        return fi if 0 != result else None

    def SM_IsSmartBatteryInstalled(self, battery_id):
        # typedef int (*PSM_IsSmartBatteryInstalled)(int batteryid);
        func = DllFunction(self._dll_obj, 'SM_IsSmartBatteryInstalled')
        result = func.call(battery_id)
        BatteryMgr._log_result(func, result)
        return result

    def SM_GetSmartBatteryStatus(self, battery_id):
        # typedef int (*PSM_GetSmartBatteryStatus)(int batteryid, SmartBatteryStatus* pOutStatus);
        ss = SmartBatteryStatus()
        func = DllFunction(self._dll_obj, 'SM_GetSmartBatteryStatus')
        result = func.call(battery_id, ctypes.byref(ss))
        BatteryMgr._log_result(func, result)
        return ss if 0 == result else None

    def SM_GetChargeStartCapacityThreshold(self, battery_id):
        # typedef int (*PSM_GetChargeStartCapacityThreshold)(int batteryid, ChargeCapacityThreshold* pOutThreshold);
        cct = ChargeCapacityThreshold()
        func = DllFunction(self._dll_obj, 'SM_GetChargeStartCapacityThreshold')
        result = func.call(battery_id, ctypes.byref(cct))
        BatteryMgr._log_result(func, result)
        return self._GetChargeCapacityThreshold(battery_id, cct) if 0 == result else None

    def SM_GetChargeStopCapacityThreshold(self, battery_id):
        # typedef int (*PSM_GetChargeStopCapacityThreshold)(int batteryid, ChargeCapacityThreshold* pOutThreshould);
        cct = ChargeCapacityThreshold()
        func = DllFunction(self._dll_obj, 'SM_GetChargeStopCapacityThreshold')
        result = func.call(battery_id, ctypes.byref(cct))
        BatteryMgr._log_result(func, result)
        return self._GetChargeCapacityThreshold(battery_id, cct) if 0 == result else None

    def SM_SetChargeStartCapacityThreshold(self, battery_id, percentage):
        # typedef int (*PSM_SetChargeStartCapacityThreshold)(int battery_id, ChargeCapacityThreshold* pOutThreshold);
        cct = ChargeCapacityThreshold()
        cct.percentage = percentage
        func = DllFunction(self._dll_obj, 'SM_SetChargeStartCapacityThreshold')
        result = func.call(battery_id, ctypes.byref(cct))
        BatteryMgr._log_result(func, result)
        self._delay_execution()
        return result

    def SM_SetChargeStopCapacityThreshold(self, battery_id, percentage):
        # typedef int (*PSM_SetChargeStopCapacityThreshold)(int battery_id, ChargeCapacityThreshold* pOutThreshold);
        cct = ChargeCapacityThreshold()
        cct.percentage = percentage
        func = DllFunction(self._dll_obj, 'SM_SetChargeStopCapacityThreshold')
        result = func.call(battery_id, ctypes.byref(cct))
        BatteryMgr._log_result(func, result)
        self._delay_execution()
        return result

    def SM_GetChargeMode(self):
        # typedef int (*PSM_GetChargeMode)();
        func = DllFunction(self._dll_obj, 'SM_GetChargeMode')
        result = func.call()
        BatteryMgr._log_result(func, result)
        return ChargeMode(result)

    def SM_SetAutoChargeMode(self):
        # typedef int (*PSM_SetAutoChargeMode)();
        func = DllFunction(self._dll_obj, 'SM_SetAutoChargeMode')
        result = func.call()
        BatteryMgr._log_result(func, result)
        self._delay_execution()
        return result

    def SM_SetNonAutoChargeMode(self, battery_id):
        # typedef int (*PSM_SetNonAutoChargeMode)(int battery_id);
        func = DllFunction(self._dll_obj, 'SM_SetNonAutoChargeMode')
        result = func.call(battery_id)
        BatteryMgr._log_result(func, result)
        self._delay_execution()
        return result

    def SM_GetDischargeStatus(self, battery_id):
        # typedef int (__fastcall *PSM_GetDischargeStatus)(int batteryid, DischargeStatus* pOutStatus);
        ds = DischargeStatus()
        func = DllFunction(self._dll_obj, 'SM_GetDischargeStatus')
        result = func.call(battery_id, ctypes.byref(ds))
        BatteryMgr._log_result(func, result)
        return ds if 0 == result else None

    def SM_SetDischargeStatus(self, battery_id, enable_discharge, enable_break_on_ac_detach):
        # typedef int (*PSM_SetDischargeStatus)(int batteryid, DischargeStatus* pStatus);
        ds = DischargeStatus()
        ds.enable_discharge = enable_discharge
        ds.enable_break_on_ac_detach = enable_break_on_ac_detach
        func = DllFunction(self._dll_obj, 'SM_SetDischargeStatus')
        result = func.call(battery_id, ctypes.byref(ds))
        BatteryMgr._log_result(func, result)
        self._delay_execution()
        return result

    # non-UEFI only
    def SM_CheckDischargePossibility(self, battery_id):
        # typedef int (__fastcall *PSM_CheckDischargePossibility)(int batteryid);
        func = DllFunction(self._dll_obj, 'SM_CheckDischargePossibility')
        result = func.call(battery_id)
        BatteryMgr._log_result(func, result)
        return result

    # non-UEFI only
    def SM_StartDischarging(self, battery_id):
        # typedef int (*PSM_StartDischarging)(int batteryid);
        func = DllFunction(self._dll_obj, 'SM_StartDischarging')
        result = func.call(battery_id)
        BatteryMgr._log_result(func, result)
        self._delay_execution()
        return result

    # non-UEFI only
    def SM_StopDischarging(self, battery_id):
        # typedef int (*PSM_StopDischarging)(int batteryid);
        func = DllFunction(self._dll_obj, 'SM_StopDischarging')
        result = func.call(battery_id)
        BatteryMgr._log_result(func, result)
        self._delay_execution()
        return result

    def SM_GetInhibitChargeStatus(self, battery_id):
        # typedef int (*PSM_GetInhibitChargeStatus)(int batteryid, InhibitChargeStatus* pOutStatus);
        ics = InhibitChargeStatus()
        func = DllFunction(self._dll_obj, 'SM_GetInhibitChargeStatus')
        result = func.call(battery_id, ctypes.byref(ics))
        BatteryMgr._log_result(func, result)
        return ics if 0 == result else None

    def SM_SetInhibitChargeStatus(self, battery_id, enable_inhibition, inhibition_period):
        # typedef int(*PSM_SetInhibitChargeStatus)(int batteryid, InhibitChargeStatus* pStatus);
        ics = InhibitChargeStatus()
        ics.enable_inhibition = enable_inhibition
        ics.inhibition_period = inhibition_period
        func = DllFunction(self._dll_obj, 'SM_SetInhibitChargeStatus')
        result = func.call(battery_id, ctypes.byref(ics))
        BatteryMgr._log_result(func, result)
        self._delay_execution()
        return result

    # non-UEFI only
    def SM_GetPeakshiftCapability(self):
        # typedef int (*PSM_GetPeakshiftCapability)();
        func = DllFunction(self._dll_obj, 'SM_GetPeakshiftCapability')
        result = func.call()
        BatteryMgr._log_result(func, result)
        return result

    # non-UEFI only
    def SM_StartPeakshiftDischarging(self, battery_id):
        # typedef int (*PSM_StartPeakshiftDischarging)(int batteryid);
        func = DllFunction(self._dll_obj, 'SM_StartPeakshiftDischarging')
        result = func.call(battery_id)
        BatteryMgr._log_result(func, result)
        self._delay_execution()
        return result

    # non-UEFI only
    def SM_StopPeakshiftDischarging(self, battery_id):
        # typedef int (*PSM_StopPeakshiftDischarging)(int batteryid);
        func = DllFunction(self._dll_obj, 'SM_StopPeakshiftDischarging')
        result = func.call(battery_id)
        BatteryMgr._log_result(func, result)
        self._delay_execution()
        return result


def print_charge_thresholds(battery_id):
    mgr = BatteryMgr()

    start_threshold = mgr.SM_GetChargeStartCapacityThreshold(battery_id)
    if start_threshold is None:
        start_threshold = 'Unknown'
    elif start_threshold >= 100:
        start_threshold = 'Unknown'
    print('Charge start threshold: %s' % str(start_threshold + 1))
    
    stop_threshold = mgr.SM_GetChargeStopCapacityThreshold(battery_id)
    if start_threshold is None:
        start_threshold = 'Unknown'
    elif start_threshold > 100:
        start_threshold = 'Unknown'
    elif (start_threshold != 0) and (stop_threshold == 0):
      stop_threshold = 100
    print('Charge stop threshold: %s' % str(stop_threshold))

def print_battery_info(battery_id):
    def print_outlined(st):
        if st is None:
            print('\tNone')
        else:
            for name, value in st.to_list_pair():
                print('\t%s:\t%s' % (name.ljust(30, ' '), value))

    mgr = BatteryMgr()

    print('Firmware info: %s' % mgr.SM_GetFirmwareInformation(battery_id))
    print('')

    smart = mgr.SM_GetSmartBatteryStatus(battery_id)
    print('Smart status: ')
    print_outlined(smart)
    print('')

    print_charge_thresholds(battery_id)
    print('')

    discharge = mgr.SM_GetDischargeStatus(battery_id)
    print('Discharge status: ')
    print_outlined(discharge)
    print('')

    chargemode = mgr.SM_GetChargeMode()
    print('Charge mode: %s' % chargemode)
    print('')

    inhibitmode = mgr.SM_GetInhibitChargeStatus(battery_id)
    print('Inhibit charge status: ')
    print_outlined(inhibitmode)
    print('')

    if 0 == mgr.SM_IsUEFI():
        print('Discharge possibility: %s' % mgr.SM_CheckDischargePossibility(battery_id))
        print('')

    if 0 == mgr.SM_IsUEFI():
        print('Peakshift capability: %s' % mgr.SM_GetPeakshiftCapability())
        print('')


def set_battery_charge_thresholds(battery_id, low_threshold, high_threshold):
    mgr = BatteryMgr()
    
    if low_threshold < 2:
        logging.error('The start charge threshold must be at least 2%')
        return
    if high_threshold - low_threshold < 4:
        logging.error('The stop charge threshold must be at least 4% above the start threshold.')
        return

    mgr.SM_SetChargeStartCapacityThreshold(battery_id, low_threshold - 1)
    mgr.SM_SetChargeStopCapacityThreshold(battery_id, high_threshold)


def start_discharging(battery_id, break_on_ac_detach):
    mgr = BatteryMgr()

    if mgr.SM_IsUEFI() == 1:
        logging.debug('The system is UEFI capable, using "SetDischargeStatus"')
        mgr.SM_SetDischargeStatus(battery_id, True, break_on_ac_detach)
    else:
        logging.debug('The system is NOT UEFI capable, using "StartDischarging"')
        mgr.SM_StartDischarging(battery_id)


def stop_discharging(battery_id):
    mgr = BatteryMgr()

    if mgr.SM_IsUEFI() == 1:
        logging.debug('The system is UEFI capable, using "SetDischargeStatus"')
        mgr.SM_SetDischargeStatus(battery_id, False, False)
    else:
        logging.debug('The system is NOT UEFI capable, using "StopDischarging"')
        mgr.SM_StopDischarging(battery_id)


def enable_inhibit_charge(battery_id, minutes):
    mgr = BatteryMgr()
    mgr.SM_SetInhibitChargeStatus(battery_id, True, minutes)


def disable_inhibit_charge(battery_id):
    mgr = BatteryMgr()
    mgr.SM_SetInhibitChargeStatus(battery_id, False, 0)


def print_usage():
    import os
    print('Usage: ' + os.path.basename(sys.argv[0]) + ' [ --debug|-d ] [ --battery|-b NUM ] command [ command_arguments ]')
    print('Where command is one of:')
    print('  `status`                  Show battery status.')
    print('')
    print('  `set-charge-thresholds`   Set battery charge thresholds')
    print('       Arguments: low high')
    print('')
    print('  `start-discharging`       Start battery discharging')
    print('       Arguments: [ break_on_ac_detach ]')
    print('')
    print('  `stop-discharging`        Start battery discharging')
    print('')
    print('  `enable-inhibit-charge`   Enable inhibit battery charge')
    print('       Arguments: minutes')
    print('')
    print('  `disable-inhibit-charge`  Disable inhibit battery charge')
    print('')
    print('---')
    print('If no battery specified the default battery #1 is used.')
    print('')


def parse_args(battery_id, args):
    command = str()

    def wrong_args(error=None):
        if error is None:
            error = 'Incorrect command line: %s %s' % str([command] + args)
        logging.error(error)
        print_usage()
        sys.exit(-1)

    if len(args) == 0:
        wrong_args('No command specified')

    command = args[0]
    args = args[1:]

    def too_much_args(count):
        if len(args) > count:
            wrong_args('Too much args (need %d): %s' % (count, str(args[1:])))

    def too_few_args(count):
        if len(args) < count:
            wrong_args('Too few args (need %d): %s' % (count, str(args[1:])))

    if command == 'status':
        too_much_args(0)
        print_battery_info(battery_id)

    elif command == 'set-charge-thresholds':
        too_much_args(2)
        too_few_args(2)
        set_battery_charge_thresholds(battery_id, int(args[0]), int(args[1]))

    elif command == 'start-discharging':
        too_much_args(1)
        start_discharging(battery_id, str2bool(args[0]) if len(args) > 0 else False)

    elif command == 'stop-discharging':
        too_much_args(0)
        stop_discharging(battery_id)

    elif command == 'enable-inhibit-charge':
        too_much_args(1)
        too_few_args(1)
        enable_inhibit_charge(battery_id, int(args[0]))

    elif command == 'disable-inhibit-charge':
        too_much_args(0)
        disable_inhibit_charge(battery_id)

    else:
        wrong_args('Unknown command: %s' % command)


# main
def main(argv):
    battery_id = 1

    try:
        import getopt
        opts, args = getopt.getopt(argv, 'db:', ['debug', 'battery'])
    except Exception as e:
        print(str(e))
        print_usage()
        sys.exit(-1)

    for opt, arg in opts:
        if opt in ('--debug', '-d'):
            global DEBUG
            DEBUG = True
        if opt in ('--battery', '-b'):
            battery_id = int(arg)

    if DEBUG:
        logging.basicConfig(level=logging.DEBUG, format='%(asctime)s %(levelname)s %(name)s: %(message)s')
    else:
        logging.basicConfig(level=logging.INFO, format='%(message)s')

    logging.debug('Python is 64 bit: %d' % is_python_64bit())
    logging.debug('Windows is 64 bit: %d' % is_windows_64bit())
    logging.debug('Using battery #%d' % battery_id)
    logging.debug('Commands: %s' % str(args))

    parse_args(battery_id, args)


if __name__ == '__main__':
    main(sys.argv[1:])
